function graph = buildGraphUni(graph_single, extract, type)

if ~exist('type','var')
    type = 0;
end

% flatten graph
offset = zeros(length(graph_single),1);
IdBW = struct('iScale',{},'iSeg',{});
nVerts = 0;
for iScale = 1:length(graph_single)
    n = length(unique(graph_single(iScale).imSeg(:)));    
    offset(iScale) = nVerts;
    IdBW = vertcat(IdBW, struct('iScale', num2cell(iScale*ones(n,1)),'iSeg', num2cell([1:n]')));    
    nVerts = nVerts+n;
end
G.IdBW=IdBW;
G.BW = graph_single;  % source
G.iVert2bw = @(i) ismember(G.BW(IdBW(i).iScale).imSeg, IdBW(i).iSeg);  % mapping to source




% define total connectivity

G.nSegs = 0;  % number of pairs
G.S = zeros(2,0);  % pairs of vertices

for iScale = 1:length(graph_single)    
    % list of directed edges
    adjmat = G.BW(iScale).adjmat;
    [R C]=find(tril(adjmat,-1)+triu(adjmat,1));
        
    Ivert = [R';C']+offset(iScale);
    
    G.S = horzcat(G.S, Ivert);
    G.nSegs = G.nSegs + size(Ivert,2);
    
    
    
    % add cross-scale connectivity here
    if type>0 && iScale < length(graph_single)
        A12 = SuperpixelAdjacency2(G.BW(iScale).imSeg, G.BW(iScale+1).imSeg);       
        [R2 C2]=find(A12);
        
        Ivert = [R2'+offset(iScale);C2'+offset(iScale+1)];
        Ivert = [Ivert flipud(Ivert)];
        
        G.S = horzcat(G.S, Ivert);
        G.nSegs = G.nSegs + size(Ivert,2);
    end
end


G.L = ones(G.nSegs,1);  % pair length (= uncoveredness) variables
G.D = zeros(149,G.nSegs);  % pair descriptors
G.Th = zeros(G.nSegs,1);
G.Succ = false(G.nSegs);  % pair-pair directed adjacency
G.Trans = zeros(G.nSegs);  % pair-pair transition cost




% loop over all pairs
fprintf('..computing %d edge features..\n',G.nSegs);
for i=1:G.nSegs
    
    % pair descriptor
    IdBW = G.IdBW(G.S(:,i));
    
    rp1=regionprops(G.BW(IdBW(1).iScale).imSeg == IdBW(1).iSeg,'Centroid');
    rp2=regionprops(G.BW(IdBW(2).iScale).imSeg == IdBW(2).iSeg,'Centroid');
    CR1 = round(rp1(1).Centroid);
    CR2 = round(rp2(1).Centroid);    
    
    bw1=ismember(G.BW(IdBW(1).iScale).imSeg, IdBW(1).iSeg);
    bw2=ismember(G.BW(IdBW(2).iScale).imSeg, IdBW(2).iSeg);

    [d,~] = extract(bw1, bw2, []);
    
    G.D(:,i) = d;
        
    % pair orientation
    th = atan2(CR2(2) - CR1(2), CR2(1) - CR1(1));
    G.Th(i) = th;
end




for i=1:G.nSegs
for j=1:G.nSegs
    % endpt = successor's startpt
    % predecessor's endpt = startpt
    G.Succ(i,j) = G.S(2,i) == G.S(1,j);
end
end
G.Opp = G.Succ & G.Succ';


% Prune out successors that change orientation too much.
for i=1:G.nSegs
for j=1:G.nSegs
    if G.Succ(i,j)
        dTh = G.Th(j) - G.Th(i);
        dTh = min(abs([dTh, dTh + 2*pi, dTh - 2*pi]));

        % threshold
        G.Succ(i,j) = dTh < pi/3;
        
        if type < 0
            G.Succ(i,j) = G.S(2,j) ~= G.S(1,i);
        end
    end
end
end


for i=1:G.nSegs
for j=1:G.nSegs
    if G.Succ(i,j)
        tr = 0;
    else
        tr = inf;
    end
    G.Trans(i,j) = tr;
end
end



graph = G;
