function parts = detectOneImage(idIm, params)

im = im2double(imread(sprintf('data/%s/%s.jpg',idIm,idIm)));
parts = [];



% function to extract a feature descriptor
imdata=load(sprintf('preprocess/%s/%s-data.mat',idIm,idIm),'image_data');
funExt = getFeatureExtractor(params, imdata.image_data);




idGraph = sprintf('detect/%s/%s_%s', idIm,idIm,params.graph.string);
if params.graph.recompute || ~exist([idGraph '.mat'],'file')
    tGraph=tic;
    
    graph = buildGraph(idIm,params,funExt);

    funEval = getFeatureEvaluator(params);
    for iScale = 1:length(graph)
    for i=1:graph(iScale).nSegs
        d = graph(iScale).D(:,i);
        v = funEval(d);
        graph(iScale).W(i) = 1-v;
    end
    end

    if strcmp(params.graph.id, 'Ternary') || strcmp(params.graph.id, 'TernaryXscale')
        fprintf('..computing edge-pair features..\n');
        for i=1:graph.nSegs
        for j=1:graph.nSegs
            if graph.Succ(i,j)
                %j is a successor of i, so any succession incurs t(i,j)

                Ivert1=graph.S(:,i);
                Ivert2=graph.S(:,j);        
                bw1 = graph.iVert2bw(Ivert1(1)) | graph.iVert2bw(Ivert1(2));
                bw2 = graph.iVert2bw(Ivert2(1)) | graph.iVert2bw(Ivert2(2));

                [d,~] = funExt(bw1,bw2,[]);
                v = funEval(d);
                graph.Trans(i,j)=1-v;
            end
        end
        end
    end
    
    t=toc(tGraph);
    save([idGraph '.mat'],'graph');
    fprintf('built graph: %f sec\n',t);
else
    load([idGraph '.mat'],'graph');
    fprintf('read %s\n',idGraph);
end
% if params.graph.vis
%     ImVis = displayGraph(im,idIm,graph,params);
%     tlWriteFigVisArray(ImVis.cellData,ImVis.funFigVis,idGraph,'png','multi');
% end



idGrouping = sprintf('detect/%s/%s_%s', idIm,idIm,params.grouping.string);

if params.grouping.recompute || ~exist([idGrouping '.mat'],'file')
    tGroup=tic;
    parts = findParts(params,graph);
    t=toc(tGroup);
    save([idGrouping '.mat'],'parts');
    fprintf('found parts: %f sec\n',t);
else
    load([idGrouping '.mat'],'parts');
    fprintf('read %s\n',idGrouping);    
end
if params.grouping.vis
    ImVis = displayParts(im,graph,parts);
    tlWriteFigVisArray(ImVis.cellData,ImVis.funFigVis,idGrouping,'png','multi');
end



end
