function [symmetry_feature, supporting_stats] = RegionSymmetryFeature9(boundary, shape_pixels, pb, dilated_pb, display_on, im)
% Fit parametric shape to boundary.

% region --> ellipse
imRegion=zeros(size(pb));
imRegion(shape_pixels)=1;
stats = regionprops(imRegion, 'Orientation', 'MajorAxisLength', 'MinorAxisLength', 'Centroid');
% convert xy output back to rc:
wInit.ac = stats(1).MajorAxisLength/2;
wInit.ar = stats(1).MinorAxisLength/2;
wInit.th = -stats(1).Orientation*(pi/180);
wInit.pc = stats(1).Centroid(1);
wInit.pr = stats(1).Centroid(2);
wInit.t = 0;
wInit.b = .000001;

lb = [.001 .001 -2*pi -inf -inf -.5 -.5]';
ub = [1000 1000 2*pi  inf  inf .5  .5]';

[R,C] = ind2sub(size(pb), boundary);  % convert list to 2D coordinates
W = ones(length(boundary),1);  % boundary points
% W = dilated_pb(boundary);  % boundary edge strengths
if isempty(W) || length(R) < 7
    wFit=[];
else
    [wFit,~]=de_fit(C,R,W,wInit,lb,ub);
end
if isempty(wFit)
    wFinal=wInit;
else
    wFinal=wFit;
end

% warp to canonical form
[R,C] = ind2sub(size(pb), boundary);
% W = dilated_pb(boundary);
W = double(dilated_pb(boundary) > 0.01);
boundary_strength = W;

s=10;
sspace=linspace(-1.5,1.5,10);
desc = de_desc(C,R,W,wFinal,sspace);

desc = desc ./ (numel(boundary) + eps);  % SPDG
major_minor_hist = reshape(desc,s,s);
minor_hist = mean(major_minor_hist, 2);
major_hist = mean(major_minor_hist, 1);
anisotropy = wFinal.ac/wFinal.ar;

symmetry_feature = [major_minor_hist(:); major_hist(:); minor_hist(:); anisotropy;];
supporting_stats.boundary_strength = boundary_strength;
supporting_stats.ellipse = wInit;
supporting_stats.dse = wFinal;
supporting_stats.boundary = boundary;

end