function parts = findParts_dig(G)

    A=.1;
    lmax=20;
    
    max_cd = 0.9;
    
    GG = G;

    nSeq =0;
    Seq=cell(0);
    CD=[];
    
    covered={};
    
    while 1
        
        [GG,seq,cd] = findPath(GG,A,lmax,nSeq+1);
        
        if cd > max_cd
            break;
        end
        
        % record next found seq
        nSeq=nSeq+1;        
        Seq{nSeq} = seq;
        CD(nSeq) = cd;
        
        iSeg = G.S(:,seq);
        covered{nSeq} = unique(iSeg(:))';
        
%         figure(5);displayGraphDigSub([],G,Seq);
        
    end

%     fprintf('%d e(v1,v2)-sequences found.\n',nSeq);
    
%     figure(5);displayGraphDigSub([],G,Seq,W);
    
    
    
    iHi = find([CD inf] > max_cd, 1) - 1;
    
    BW = false([size(G.BW(1).imSeg) iHi]);
    scores = zeros(iHi,1);
    Iverts = cell(iHi,1);
    
    for i=1:iHi
        Ivert = G.S(:,Seq{i});
        Ivert = unique(Ivert(:));        
        
        for j=1:length(Ivert)
            BW(:,:,i) = BW(:,:,i) | G.iVert2bw(Ivert(j));
        end        
        
        Iverts{i} = Ivert;
        scores(i) = exp(-CD(i));
    end
    
    
    parts.imRegions = BW;
    parts.scores = scores;
    parts.Iverts = Iverts;
    
    parts.Seq = Seq(1:iHi);
    parts.CD = CD(1:iHi);

end


function [G,seq,w] = findPath(G, A, lmax, iTrial)

nSegs = G.nSegs;

min_cd=inf;

W = inf(nSegs,lmax+1);
T = zeros(nSegs,lmax+1);
pq = [];
for s=1:nSegs
    sub = [s;G.L(s)];
    w=G.W(s);
    W(sub(1),sub(2)+1) = w;
    pq=enq(pq,sub);    
    
    cd=(A+w)/sub(2);
    if cd < min_cd
        min_cd=cd;
        min_cd_sub=sub;
    end
end


maxpq=length(pq);

iIt=0;
nlmaxreached=0;

extend = true;
while extend
    iIt=iIt+1;
    
    % extract density-lightest curveslot
    [pq,sub]=deq(pq,W);
    s=sub(1);
    l=sub(2);

    % get successors
    Suc = find(G.Succ(s,:));
    
    
%     str=sprintf('it#%d dequeue C(%d,%d) at %0.4fwd  ->|PQ|=%d', ...
%         iIt,s,l,W(s,l+1)/l,length(pq));
%     figure(5);displayGraphDigSub([],G,{follow(G,T,s,l)});
%     title(str);
%     fprintf('%s\n',str);
    
    if l == lmax
        Suc = [];
        nlmaxreached=nlmaxreached+1;
    end

    
    for q = Suc
        % now considering candidate for (q,k=l+1) ...
        
        v = W(s,l+1) + G.Trans(s,q) + G.W(q);
        k = l + G.L(q);

        if k>lmax
            
            continue;
        end
        
%         fprintf(' propose C(%d,%d)+S(%d) at %0.4fwd vs %0.4fwd', ...
%             s,l,q, v/k, W(q,k+1)/k);
        
        sub=[q;k];

        if v < W(sub(1),sub(2)+1)    
            W(sub(1),sub(2)+1)=v;
            T(sub(1),sub(2)+1)=s;
            pq=enq(pq,sub);
            
            
            cd=(A+v)/k;
            if cd < min_cd
                min_cd=cd;
                min_cd_sub=sub;
            end
            
            
%             fprintf('  ==>  enqueue ->|PQ|=%d\n',length(pq));
%         else
%             fprintf('\n');
        end
    end
    
    
    if length(pq)>maxpq
        maxpq=length(pq);
    end
    
    
    
    
    WD=W(sub2ind(size(W),pq(1,:),pq(2,:)+1))./pq(2,:);
    [min_wd, ~] = min(WD(:));    
    
    extend = min_cd > min_wd;
    
%     fprintf(' (minCostDensity=%0.4f  >  minPQweightDensity=%0.4f) is %d\n', minCostDensity, minPQweightDensity, extend);
end


if min_cd == inf
    seq = [];
else
    s=min_cd_sub(1);
    l=min_cd_sub(2);
    seq=follow(G,T,s,l);
end

G.L(seq)=0;
G.L(any(G.Opp(seq,:),1))=0;

% fprintf('(lmax=%d reached %d times\n',lmax,nlmaxreached);
% fprintf('#%0d %0.4fs |seq|=%d %0.4fcd %dits max|Q|=%d\n',iTrial,toc(tic1),length(seq),min_cd,iIt,maxpq);

w=min_cd;

end

function seq = follow(G,T,s,l)
    seq=[];
    k=l;
    while s>0    
        seq = [s seq];
        s_pred=T(s,k+1); % get predecessor    
        k = k - G.L(s); % get next length
        s=s_pred;
    end
end


function Qout = enq(Qin, e)
    Qout = [Qin e];
end

function [Qout, sl] = deq(Qin, W)

    S=Qin(1,:);
    L=Qin(2,:);

    order = W(sub2ind(size(W),S,L+1))./L;
    [~,i]=min(order);
    sl=Qin(:,i);
    Qout = [Qin(:,1:i-1) Qin(:,i+1:end)];

end
