function funExt = getFeatureExtractor(params, image_data)


switch params.feature.id
    case 'Ellipse'
	funExt = @(bw1,bw2,diBW) extFeatIccv09(bw1,bw2,image_data);
    case 'DefDisc'
    funExt = @(bw1,bw2,diBW) extFeatDil(bw1,bw2,image_data,ones(2));
end

end


function [d149,s] = extFeatIccv09(bw1,bw2,image_data)
        [app_feature, boundary, shape_pixels] = AppFeature(bw1,bw2,image_data);
        [symmetry_feature, supporting_stats] = RegionSymmetryFeature4(boundary, shape_pixels, image_data.pb, image_data.dilated_pb, 0);
        d149 = [symmetry_feature; app_feature];
        s = supporting_stats;
end

function [d149,s] = extFeatDil(bw1,bw2,image_data,el)
        [app_feature, boundary, shape_pixels] = AppFeature(bw1,bw2,image_data);
        dilated_pb = imdilate(image_data.pb,el);
        [symmetry_feature, supporting_stats] = RegionSymmetryFeature9(boundary, shape_pixels, image_data.pb, dilated_pb, 0);
        d149 = [symmetry_feature; app_feature];
        s = supporting_stats;
end


function [app_feature, boundary, shape_pixels] = AppFeature(BW1,BW2,image_data)

bw=BW1|BW2;
boundary = RegionBoundary(bw);
shape_pixels = find(bw);

        num_bins_color = 10;
        num_bins_texture = 32;
        tex_range = (1:(num_bins_texture+1)) - 0.5;


        color_hist1 = RegionColorHistogram(BW1, image_data.img_h, image_data.img_s, image_data.img_v, num_bins_color);
        color_hist2 = RegionColorHistogram(BW2, image_data.img_h, image_data.img_s, image_data.img_v, num_bins_color);
        mean_r1 = mean(image_data.img_r(BW1));
        mean_g1 = mean(image_data.img_g(BW1));
        mean_b1 = mean(image_data.img_b(BW1));
        var_r1 = var(image_data.img_r(BW1));
        var_g1 = var(image_data.img_g(BW1));
        var_b1 = var(image_data.img_b(BW1));
        mean_r2 = mean(image_data.img_r(BW2));
        mean_g2 = mean(image_data.img_g(BW2));
        mean_b2 = mean(image_data.img_b(BW2));
        var_r2 = var(image_data.img_r(BW2));
        var_g2 = var(image_data.img_g(BW2));
        var_b2 = var(image_data.img_b(BW2));

        mean_cos1 = mean(image_data.cos_hs(BW1));
        mean_sin1 = mean(image_data.sin_hs(BW1));
        mean_v1 = mean(image_data.img_v(BW1));
        var_cos1 = var(image_data.cos_hs(BW1));
        var_sin1 = var(image_data.sin_hs(BW1));
        var_v1 = var(image_data.img_v(BW1));
        mean_cos2 = mean(image_data.cos_hs(BW2));
        mean_sin2 = mean(image_data.sin_hs(BW2));
        mean_v2 = mean(image_data.img_v(BW2));
        var_cos2 = var(image_data.cos_hs(BW2));
        var_sin2 = var(image_data.sin_hs(BW2));
        var_v2 = var(image_data.img_v(BW2));

        mean_color1 = [mean_r1, mean_g1, mean_b1];
        var_color1 = [var_r1, var_g1, var_b1];
        mean_color2 = [mean_r2, mean_g2, mean_b2];
        var_color2 = [var_r2, var_g2, var_b2];

        mean_color_hsv1 = [mean_cos1, mean_sin1, mean_v1];
        var_color_hsv1 = [var_cos1, var_sin1, var_v1];
        mean_color_hsv2 = [mean_cos2, mean_sin2, mean_v2];
        var_color_hsv2 = [var_cos2, var_sin2, var_v2];

        color_affinity_h = AffinityColorHistogram(color_hist1(1:num_bins_color), color_hist2(1:num_bins_color));
        color_affinity_s = AffinityColorHistogram(color_hist1(num_bins_color+(1:num_bins_color)), color_hist2(num_bins_color+(1:num_bins_color)));
        color_affinity_v = AffinityColorHistogram(color_hist1(2*num_bins_color+(1:num_bins_color)), color_hist2(2*num_bins_color+(1:num_bins_color)));
        dark1 = 1-mean_v1;
        dark2 = 1-mean_v2;
        is_bright = (1-dark1)*(1-dark2);
        similar_brightness = color_affinity_v;

        color_feature = [abs(mean_color1 - mean_color2), ...
            abs(var_color1 - var_color2), ...
            var_color1, var_color2, ...
            abs(mean_color_hsv1 - mean_color_hsv2), ...
            abs(var_color_hsv1 - var_color_hsv2), ...
            var_color_hsv1, var_color_hsv2, ...
            is_bright*color_affinity_h, is_bright*color_affinity_s, (1-is_bright)*similar_brightness]';

        tex_hist1 = RegionTextureHistogram(BW1, image_data.tmap, tex_range);
        tex_hist2 = RegionTextureHistogram(BW2, image_data.tmap, tex_range);

        texture_affinity = AffinityTextureHistogram(tex_hist1(:), tex_hist2(:));


app_feature = [color_feature; texture_affinity];

end
